@@ -11,6 +11,7 @@ module Language.Futhark.TypeChecker.Unify
     Rigidity (..),
     RigidSource (..),
     BreadCrumbs,
+    sizeFree,
     noBreadCrumbs,
     hasNoBreadCrumbs,
     dimNotes,
@@ -30,14 +31,18 @@ where
 
 import Control.Monad
 import Control.Monad.Except
+import Control.Monad.Identity
+import Control.Monad.Reader
 import Control.Monad.State
 import Data.List qualified as L
 import Data.Map.Strict qualified as M
 import Data.Maybe
 import Data.Set qualified as S
 import Data.Text qualified as T
+import Futhark.Util (topologicalSort)
 import Futhark.Util.Pretty
 import Language.Futhark
+import Language.Futhark.Traversals
 import Language.Futhark.TypeChecker.Monad hiding (BoundV)
 import Language.Futhark.TypeChecker.Types
 
@@ -616,6 +621,74 @@ scopeCheck usage bcs vn max_lvl tp = do
           <+> dquotes (prettyName v)
           <+> "is rigidly bound in a deeper scope."
 
+-- Expressions witnessed by type, topologically sorted.
+topWit :: TypeBase Exp u -> [Exp]
+topWit = topologicalSort depends . witnessedExps
+  where
+    witnessedExps t = execState (traverseDims onDim t) mempty
+      where
+        onDim _ PosImmediate e = modify (e :)
+        onDim _ _ _ = pure ()
+    depends a b = any (sameExp b) $ subExps a
+
+sizeFree ::
+  (MonadUnify m) =>
+  SrcLoc ->
+  (Exp -> Maybe VName) ->
+  TypeBase Size u ->
+  m (TypeBase Size u, [VName])
+sizeFree tloc expKiller orig_t = do
+  runReaderT (toBeReplaced orig_t $ onType orig_t) mempty `runStateT` mempty
+  where
+    lookReplacement e repl = snd <$> L.find (sameExp e . fst) repl
+    expReplace mapping e
+      | Just e' <- lookReplacement e mapping = e'
+      | otherwise = runIdentity $ astMap mapper e
+      where
+        mapper = identityMapper {mapOnExp = pure . expReplace mapping}
+
+    replacing e = do
+      e' <- asks (`expReplace` e)
+      case expKiller e' of
+        Nothing -> pure e'
+        Just cause -> do
+          vn <- lift $ lift $ newRigidDim tloc (RigidOutOfScope (locOf e) cause) "d"
+          modify (vn :)
+          pure $ sizeFromName (qualName vn) (srclocOf e)
+
+    toBeReplaced t m' = foldl f m' $ topWit t
+      where
+        f m e = do
+          e' <- replacing e
+          local ((e, e') :) m
+
+    onScalar (Record fs) =
+      Record <$> traverse onType fs
+    onScalar (Sum cs) =
+      Sum <$> (traverse . traverse) onType cs
+    onScalar (Arrow as pn d argT (RetType dims retT)) = do
+      argT' <- onType argT
+      old_bound <- get
+      retT' <- toBeReplaced retT $ onType retT
+      rl <- state $ L.partition (`notElem` old_bound)
+      let dims' = dims <> rl
+      pure $ Arrow as pn d argT' (RetType dims' retT')
+    onScalar (TypeVar u v args) =
+      TypeVar u v <$> mapM onTypeArg args
+      where
+        onTypeArg (TypeArgDim d) = TypeArgDim <$> replacing d
+        onTypeArg (TypeArgType ty) = TypeArgType <$> onType ty
+    onScalar (Prim pt) = pure $ Prim pt
+
+    onType ::
+      (MonadUnify m) =>
+      TypeBase Size u ->
+      ReaderT [(Exp, Exp)] (StateT [VName] m) (TypeBase Size u)
+    onType (Array u shape scalar) =
+      Array u <$> traverse replacing shape <*> onScalar scalar
+    onType (Scalar ty) =
+      Scalar <$> onScalar ty
+
 linkVarToType ::
   (MonadUnify m) =>
   UnifySizes m ->
@@ -633,20 +706,24 @@ linkVarToType onDims usage bound bcs vn lvl tp_unnorm = do
   occursCheck usage bcs vn tp
   scopeCheck usage bcs vn lvl tp
 
-  constraints <- getConstraints
   let link = do
         let (witnessed, not_witnessed) = determineSizeWitnesses tp
             used v = v `S.member` witnessed || v `S.member` not_witnessed
-            ext = filter used bound
-        case filter (`notElem` witnessed) ext of
-          [] ->
-            modifyConstraints $
-              M.insert vn (lvl, Constraint (RetType ext tp) usage)
-          problems ->
-            unifyError usage mempty bcs . withIndexLink "unify-param-existential" $
-              "Parameter(s) "
-                <> commasep (map (dquotes . prettyName) problems)
-                <> " used as size(s) would go out of scope."
+            (ext_witnessed, ext_not_witnessed) =
+              L.partition (`elem` witnessed) $ filter used bound
+
+            -- Any size that uses an ext_not_witnessed variable must
+            -- be replaced with a fresh existential.
+            problematic e =
+              L.find (`elem` ext_not_witnessed) $
+                S.toList $
+                  fvVars $
+                    freeInExp e
+
+        (tp', ext_new) <- sizeFree (srclocOf usage) problematic tp
+
+        modifyConstraints $
+          M.insert vn (lvl, Constraint (RetType (ext_new <> ext_witnessed) tp') usage)
 
   let unliftedBcs unlifted_usage =
         breadCrumb
@@ -658,6 +735,7 @@ linkVarToType onDims usage bound bcs vn lvl tp_unnorm = do
           )
           bcs
 
+  constraints <- getConstraints
   case snd <$> M.lookup vn constraints of
     Just (NoConstraint Unlifted unlift_usage) -> do
       link
